@@ -17,7 +17,7 @@ setMethod("interpolate", signature(object="SpatRaster"),
 		nc <- ncol(out)
 		testrow <- round(0.5*nrow(object))
 		xy <- xyFromCell(out, cellFromRowCol(out, testrow, 1):cellFromRowCol(out, testrow, min(nc, 500)))
-		names(xy) <- xyNames
+		colnames(xy) <- xyNames
 		if (hv) { 
 			d <- readValues(object, testrow, 1, 1, nc, TRUE, TRUE)
 			xy <- cbind(xy, d)
@@ -32,13 +32,13 @@ setMethod("interpolate", signature(object="SpatRaster"),
 		b <- writeStart(out, filename, overwrite, wopt)
 		for (i in 1:b$n) {
 			xy <- xyFromCell(out, cellFromRowCol(out, b$row[i], 1):cellFromRowCol(out, b$row[i]+b$nrows[i]-1, nc))
-			names(xy) <- xyNames
+			colnames(xy) <- xyNames
 			if (hv) { 
 				d <- readValues(object, b$row[i], b$nrows[i], 1, nc, TRUE, TRUE)
 				xy <- cbind(xy, d)
 			}
 			v <- .runModel(model, fun, xy, nl, const, (na.rm & hv), index, ...)
-			writeVals(out, v, b$row[i], b$nrows[i])
+			writeValues(out, v, b$row[i], b$nrows[i])
 		}
 		if (hv) readStop(object)
 		out <- writeStop(out)
