@@ -67,7 +67,7 @@ SpatPart::SpatPart(std::vector<double> X, std::vector<double> Y) {
 }
 
 
-SpatGeom::SpatGeom() {};
+SpatGeom::SpatGeom() {}
 
 SpatGeom::SpatGeom(SpatPart p) {
 	parts.push_back(p);
@@ -125,16 +125,16 @@ SpatPart SpatGeom::getPart(unsigned i) {
 	return parts[i];
 }
 
-SpatVector::SpatVector() {};
+SpatVector::SpatVector() {}
 
 SpatVector::SpatVector(SpatGeom g) {
 	addGeom(g);
-};
+}
 
 /*
 SpatVector::SpatVector(const SpatVector &x) {
-	lyr.srs = x.lyr.srs;
-	lyr.df = SpatDataFrame(x.lyr.df);	
+	srs = x.srs;
+	df = SpatDataFrame(x.df);	
 }
 */
 
@@ -145,83 +145,83 @@ SpatVector::SpatVector(SpatExtent e, std::string crs) {
 	SpatGeom g(p);
 	setGeom(g);
 	setSRS( {crs});
-};
+}
 
 
 std::vector<double> SpatVector::getDv(unsigned i) {
-	return lyr.df.getD(i);
+	return df.getD(i);
 }
 
 std::vector<long> SpatVector::getIv(unsigned i){
-	return lyr.df.getI(i);
+	return df.getI(i);
 }
 
 std::vector<std::string> SpatVector::getSv(unsigned i){
-	return lyr.df.getS(i);
+	return df.getS(i);
 }
 
 std::vector<unsigned> SpatVector::getItype(){
-	return lyr.df.itype;
+	return df.itype;
 }
 
 std::vector<unsigned> SpatVector::getIplace(){
-	return lyr.df.iplace;
+	return df.iplace;
 }
 
 
 std::vector<std::string> SpatVector::get_names(){
-	return lyr.df.get_names();
+	return df.get_names();
 }
 
 void SpatVector::set_names(std::vector<std::string> s){
-	lyr.df.set_names(s);
+	df.set_names(s);
 }
 
 unsigned SpatVector::ncol() {
-	return lyr.df.ncol();
+	return df.ncol();
 }
 
 unsigned SpatVector::nrow() {
-	return lyr.geoms.size();
+	return geoms.size();
 }
 
 size_t SpatVector::size() {
-	return lyr.geoms.size();
+	return geoms.size();
 }
 
 bool SpatVector::is_lonlat() {
-	return lyr.srs.is_lonlat();
-};
+	return srs.is_lonlat();
+}
 
 bool SpatVector::could_be_lonlat() {
 	SpatExtent e = getExtent();
-	return lyr.srs.could_be_lonlat(e);
-};
+	return srs.could_be_lonlat(e);
+}
 
 
 SpatExtent SpatVector::getExtent(){
-	return lyr.extent;
+	return extent;
 }
 
 
 /*
 void SpatVector::setPRJ(std::string PRJ){
-	lyr.crs[0] = PRJ;
+	crs[0] = PRJ;
 }
 
 std::string SpatVector::getPRJ(){
-	return lyr.crs[0];
+	return crs[0];
 }
 */
 
 std::string SpatVector::type(){
 	if (size() == 0) {
 		return "none";
-	} else if (lyr.geoms[0].gtype == points) {
+	} else if (geoms[0].gtype == points) {
 		return "points";
-	} else if (lyr.geoms[0].gtype == lines) {
+	} else if (geoms[0].gtype == lines) {
 		return "lines";
-	} else if (lyr.geoms[0].gtype == polygons) {
+	} else if (geoms[0].gtype == polygons) {
 		return "polygons";
 	} else {
 		return("unknown");
@@ -231,23 +231,23 @@ std::string SpatVector::type(){
 
 
 SpatGeom SpatVector::getGeom(unsigned i) {
-	return lyr.geoms[i];
+	return geoms[i];
 }
 
 bool SpatVector::addGeom(SpatGeom p) {
-	lyr.geoms.push_back(p);
-	if (lyr.geoms.size() > 1) {
-		lyr.extent.unite(p.extent);
+	geoms.push_back(p);
+	if (geoms.size() > 1) {
+		extent.unite(p.extent);
 	} else {
-		lyr.extent = p.extent;
+		extent = p.extent;
 	}
 	return true;
 }
 
 bool SpatVector::setGeom(SpatGeom p) {
-	lyr.geoms.resize(1);
-	lyr.geoms[0] = p;
-	lyr.extent = p.extent;
+	geoms.resize(1);
+	geoms[0] = p;
+	extent = p.extent;
 	return true;
 }
 
@@ -425,26 +425,26 @@ SpatVector SpatVector::subset_rows(std::vector<int> range) {
 	}
 
 	for (size_t i=0; i < r.size(); i++) {
-		out.addGeom( lyr.geoms[r[i]] );
+		out.addGeom( geoms[r[i]] );
 	}
-	out.lyr.srs = lyr.srs;
-	out.lyr.df = lyr.df.subset_rows(r);
+	out.srs = srs;
+	out.df = df.subset_rows(r);
 	return out;
-};
+}
 
 
 SpatVector SpatVector::subset_rows(int i) {
 	std::vector<int> range(1, i);
 	SpatVector out = subset_rows(range);
 	return out;
-};
+}
 
 
 SpatVector SpatVector::subset_cols(std::vector<int> range) {
 	SpatVector out = *this;
-	//out.lyr.geoms = lyr.geoms;
-	//out.lyr.crs = lyr.crs;
-	//out.lyr.extent = lyr.extent;
+	//out.geoms = geoms;
+	//out.crs = crs;
+	//out.extent = extent;
 	int nc = ncol();
 
 	std::vector<unsigned> r;
@@ -453,16 +453,16 @@ SpatVector SpatVector::subset_cols(std::vector<int> range) {
 			r.push_back(range[i]);
 		}
 	}
-	out.lyr.df = lyr.df.subset_cols(r);
+	out.df = df.subset_cols(r);
 	return out;
-};
+}
 
 
 SpatVector SpatVector::subset_cols(int i) {
 	std::vector<int> range(1, i);
 	SpatVector out = subset_cols(range);
 	return out;
-};
+}
 
 
 
